@@ -6,17 +6,18 @@ use axes_common::*;
 use coordinates::*;
 use datatype::*;
 use options::*;
+use util::OneWayOwned;
 use writer::Writer;
 
 /// 2D axes that is used for drawing 2D plots
-pub struct Axes2D<'l>
+pub struct Axes2D
 {
-	common: AxesCommonData<'l>,
+	common: AxesCommonData,
 }
 
-impl<'m> Axes2D<'m>
+impl Axes2D
 {
-	pub(crate) fn new() -> Axes2D<'m>
+	pub(crate) fn new() -> Axes2D
 	{
 		Axes2D { common: AxesCommonData::new() }
 	}
@@ -31,7 +32,7 @@ impl<'m> Axes2D<'m>
 	///      * `Color` - Specifies the color of the border
 	///      * `LineStyle` - Specifies the style of the border
 	///      * `LineWidth` - Specifies the width of the border
-	pub fn set_border<'l>(&'l mut self, front: bool, locations: &[BorderLocation2D], options: &[PlotOption<'m>]) -> &'l mut Self
+	pub fn set_border<'l>(&'l mut self, front: bool, locations: &[BorderLocation2D], options: &[PlotOption<&str>]) -> &'l mut Self
 	{
 		{
 			let c = &mut self.common.commands as &mut Writer;
@@ -51,15 +52,16 @@ impl<'m> Axes2D<'m>
 				" back "
 			});
 
-			AxesCommonData::write_color_options(c, options, Some("black"));
-			AxesCommonData::write_line_options(c, options);
+			let options: Vec<PlotOption<String>> = options.to_one_way_owned();
+			AxesCommonData::write_color_options(c, &options, Some("black"));
+			AxesCommonData::write_line_options(c, &options);
 
 			c.write_str("\n");
 		}
 		self
 	}
 
-	fn set_axis_common<'l>(&'l mut self, axis: &str, show: bool, options: &[PlotOption<'m>]) -> &'l mut Self
+	fn set_axis_common<'l>(&'l mut self, axis: &str, show: bool, options: &[PlotOption<&str>]) -> &'l mut Self
 	{
 		{
 			let c = &mut self.common.commands as &mut Writer;
@@ -68,8 +70,10 @@ impl<'m> Axes2D<'m>
 				c.write_str("set ");
 				c.write_str(axis);
 				c.write_str("zeroaxis ");
-				AxesCommonData::write_color_options(c, options, Some("black"));
-				AxesCommonData::write_line_options(c, options);
+
+				let options: Vec<PlotOption<String>> = options.to_one_way_owned();
+				AxesCommonData::write_color_options(c, &options, Some("black"));
+				AxesCommonData::write_line_options(c, &options);
 			}
 			else
 			{
@@ -88,17 +92,17 @@ impl<'m> Axes2D<'m>
 	/// # Arguments
 	///
 	/// * `show` - Whether or not draw the axis
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the axis. Relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the axis. Relevant options are:
 	///      * `Color` - Specifies the color of the border
 	///      * `LineStyle` - Specifies the style of the border
 	///      * `LineWidth` - Specifies the width of the border
-	pub fn set_x_axis<'l>(&'l mut self, show: bool, options: &[PlotOption<'m>]) -> &'l mut Self
+	pub fn set_x_axis<'l>(&'l mut self, show: bool, options: &[PlotOption<&str>]) -> &'l mut Self
 	{
 		self.set_axis_common("x", show, options)
 	}
 
 	/// Like `set_x_axis` but for the y axis.
-	pub fn set_y_axis<'l>(&'l mut self, show: bool, options: &[PlotOption<'m>]) -> &'l mut Self
+	pub fn set_y_axis<'l>(&'l mut self, show: bool, options: &[PlotOption<&str>]) -> &'l mut Self
 	{
 		self.set_axis_common("y", show, options)
 	}
@@ -109,13 +113,13 @@ impl<'m> Axes2D<'m>
 	/// * `y1` - Y coordinate of the arrow start
 	/// * `x2` - X coordinate of the arrow end
 	/// * `y2` - Y coordinate of the arrow end
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the arrowhead and arrow shaft. Relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the arrowhead and arrow shaft. Relevant options are:
 	///      * `ArrowType` - Specifies the style of the arrow head (or an option to omit it)
 	///      * `ArrowSize` - Sets the size of the arrow head (in graph units)
 	///      * `Color` - Specifies the color of the arrow
 	///      * `LineStyle` - Specifies the style of the arrow shaft
 	///      * `LineWidth` - Specifies the width of the arrow shaft
-	pub fn arrow<'l>(&'l mut self, x1: Coordinate, y1: Coordinate, x2: Coordinate, y2: Coordinate, options: &[PlotOption<'m>])
+	pub fn arrow<'l>(&'l mut self, x1: Coordinate, y1: Coordinate, x2: Coordinate, y2: Coordinate, options: &[PlotOption<&str>])
 		-> &'l mut Self
 	{
 		{
@@ -148,8 +152,9 @@ impl<'m> Axes2D<'m>
 			}
 			c.write_str(",12");
 
-			AxesCommonData::write_color_options(c, options, Some("black"));
-			AxesCommonData::write_line_options(c, options);
+			let options: Vec<PlotOption<String>> = options.to_one_way_owned();
+			AxesCommonData::write_color_options(c, &options, Some("black"));
+			AxesCommonData::write_line_options(c, &options);
 
 			c.write_str("\n");
 		}
@@ -166,7 +171,7 @@ impl<'m> Axes2D<'m>
 	///     * `TextColor`
 	///     * `TextAlign(AlignLeft)`
 	///     * `TextAlign(AlignRight)`
-	pub fn set_legend<'l>(&'l mut self, x: Coordinate, y: Coordinate, legend_options: &[LegendOption], text_options: &[LabelOption])
+	pub fn set_legend<'l>(&'l mut self, x: Coordinate, y: Coordinate, legend_options: &[LegendOption<&str>], text_options: &[LabelOption<&str>])
 		-> &'l mut Self
 	{
 		{
@@ -291,15 +296,15 @@ impl<'m> Axes2D<'m>
 	/// # Arguments
 	/// * `x` - x values
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `LineWidth` - Sets the width of the line
 	///     * `LineStyle` - Sets the style of the line
 	///     * `Color` - Sets the color
-	pub fn lines<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+	pub fn lines<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<&str>])
 		-> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot2(Lines, x, y, options.to_vec()));
+		self.common.elems.push(PlotElement::new_plot2(Lines, x, y, options.to_one_way_owned()));
 		self
 	}
 
@@ -307,15 +312,15 @@ impl<'m> Axes2D<'m>
 	/// # Arguments
 	/// * `x` - x values
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `PointSymbol` - Sets symbol for each point
 	///     * `PointSize` - Sets the size of each point
 	///     * `Color` - Sets the color
-	pub fn points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+	pub fn points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<&str>])
 		-> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot2(Points, x, y, options.to_vec()));
+		self.common.elems.push(PlotElement::new_plot2(Points, x, y, options.to_one_way_owned()));
 		self
 	}
 
@@ -323,11 +328,11 @@ impl<'m> Axes2D<'m>
 	/// # Arguments
 	/// * `x` - x values
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element
-	pub fn lines_points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element
+	pub fn lines_points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<&str>])
 		-> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot2(LinesPoints, x, y, options.to_vec()));
+		self.common.elems.push(PlotElement::new_plot2(LinesPoints, x, y, options.to_one_way_owned()));
 		self
 	}
 
@@ -351,10 +356,12 @@ impl<'m> Axes2D<'m>
 		Txe: DataType,
 		XE: IntoIterator<Item = Txe>,
 	>(
-		&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption<'m>]
+		&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption<&str>]
 	) -> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot3(XErrorBars, x, y, x_error, options.to_vec()));
+		self.common
+			.elems
+			.push(PlotElement::new_plot3(XErrorBars, x, y, x_error, options.to_one_way_owned()));
 		self
 	}
 
@@ -364,7 +371,7 @@ impl<'m> Axes2D<'m>
 	/// * `x` - x values
 	/// * `y` - y values
 	/// * `y_error` - Errors associated with the y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `PointSymbol` - Sets symbol for each point
 	///     * `PointSize` - Sets the size of each point
@@ -378,10 +385,12 @@ impl<'m> Axes2D<'m>
 		Tye: DataType,
 		YE: IntoIterator<Item = Tye>,
 	>(
-		&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption<'m>]
+		&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption<&str>]
 	) -> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot3(YErrorBars, x, y, y_error, options.to_vec()));
+		self.common
+			.elems
+			.push(PlotElement::new_plot3(YErrorBars, x, y, y_error, options.to_one_way_owned()));
 		self
 	}
 
@@ -407,10 +416,12 @@ impl<'m> Axes2D<'m>
 		Txe: DataType,
 		XE: IntoIterator<Item = Txe>,
 	>(
-		&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption<'m>]
+		&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption<&str>]
 	) -> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot3(XErrorLines, x, y, x_error, options.to_vec()));
+		self.common
+			.elems
+			.push(PlotElement::new_plot3(XErrorLines, x, y, x_error, options.to_one_way_owned()));
 		self
 	}
 
@@ -420,7 +431,7 @@ impl<'m> Axes2D<'m>
 	/// * `x` - x values
 	/// * `y` - y values
 	/// * `y_error` - Errors associated with the y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `PointSymbol` - Sets symbol for each point
 	///     * `PointSize` - Sets the size of each point
@@ -436,10 +447,12 @@ impl<'m> Axes2D<'m>
 		Tye: DataType,
 		YE: IntoIterator<Item = Tye>,
 	>(
-		&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption<'m>]
+		&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption<&str>]
 	) -> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot3(YErrorLines, x, y, y_error, options.to_vec()));
+		self.common
+			.elems
+			.push(PlotElement::new_plot3(YErrorLines, x, y, y_error, options.to_one_way_owned()));
 		self
 	}
 
@@ -450,7 +463,7 @@ impl<'m> Axes2D<'m>
 	/// * `x` - x values
 	/// * `y_lo` - Bottom y values
 	/// * `y_hi` - Top y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `FillRegion` - Specifies the region between the two curves to fill
 	///     * `Color` - Sets the color of the filled region
@@ -464,10 +477,12 @@ impl<'m> Axes2D<'m>
 		Tyh: DataType,
 		YH: IntoIterator<Item = Tyh>,
 	>(
-		&'l mut self, x: X, y_lo: YL, y_hi: YH, options: &[PlotOption<'m>]
+		&'l mut self, x: X, y_lo: YL, y_hi: YH, options: &[PlotOption<&str>]
 	) -> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot3(FillBetween, x, y_lo, y_hi, options.to_vec()));
+		self.common
+			.elems
+			.push(PlotElement::new_plot3(FillBetween, x, y_lo, y_hi, options.to_one_way_owned()));
 		self
 	}
 
@@ -476,17 +491,17 @@ impl<'m> Axes2D<'m>
 	/// # Arguments
 	/// * `x` - x values (center of the box)
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `LineWidth` - Sets the width of the border
 	///     * `LineStyle` - Sets the style of the border
 	///     * `BorderColor` - Sets the color of the border
 	///     * `Color` - Sets the color of the box fill
 	///     * `FillAlpha` - Sets the transparency of the box fill
-	pub fn boxes<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+	pub fn boxes<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<&str>])
 		-> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot2(Boxes, x, y, options.to_vec()));
+		self.common.elems.push(PlotElement::new_plot2(Boxes, x, y, options.to_one_way_owned()));
 		self
 	}
 
@@ -496,7 +511,7 @@ impl<'m> Axes2D<'m>
 	/// * `x` - x values (center of the box)
 	/// * `y` - y values
 	/// * `w` - Box width values
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `LineWidth` - Sets the width of the border
 	///     * `LineStyle` - Sets the style of the border
@@ -512,10 +527,10 @@ impl<'m> Axes2D<'m>
 		Tw: DataType,
 		W: IntoIterator<Item = Tw>,
 	>(
-		&'l mut self, x: X, y: Y, w: W, options: &[PlotOption<'m>]
+		&'l mut self, x: X, y: Y, w: W, options: &[PlotOption<&str>]
 	) -> &'l mut Self
 	{
-		self.common.elems.push(PlotElement::new_plot3(Boxes, x, y, w, options.to_vec()));
+		self.common.elems.push(PlotElement::new_plot3(Boxes, x, y, w, options.to_one_way_owned()));
 		self
 	}
 
@@ -528,39 +543,45 @@ impl<'m> Axes2D<'m>
 	/// * `num_cols` - Number of columns in the data array
 	/// * `dimensions` - Optional X and Y coordinates of the first and last data points (with the rest of the coordinates spaced evenly between).
 	///                  By default this will be `(0, 0)` and `(num_rows - 1, num_cols - 1)`.
-	/// * `options` - Array of PlotOption<'m> controlling the appearance of the surface. Relevant options are:
+	/// * `options` - Array of PlotOption<&str> controlling the appearance of the surface. Relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
-	pub fn image<'l, T: DataType, X: IntoIterator<Item = T>>(&'l mut self, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>, options: &[PlotOption<'m>])
+	pub fn image<'l, T: DataType, X: IntoIterator<Item = T>>(&'l mut self, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>, options: &[PlotOption<&str>])
 		-> &'l mut Self
 	{
-		self.common
-			.elems
-			.push(PlotElement::new_plot_matrix(Image, false, mat, num_rows, num_cols, dimensions, options.to_vec()));
+		self.common.elems.push(PlotElement::new_plot_matrix(
+			Image,
+			false,
+			mat,
+			num_rows,
+			num_cols,
+			dimensions,
+			options.to_one_way_owned(),
+		));
 		self
 	}
 }
 
-impl<'l> AxesCommonPrivate<'l> for Axes2D<'l>
+impl AxesCommonPrivate for Axes2D
 {
-	fn get_common_data_mut(&mut self) -> &mut AxesCommonData<'l>
+	fn get_common_data_mut(&mut self) -> &mut AxesCommonData
 	{
 		&mut self.common
 	}
 
-	fn get_common_data(&self) -> &AxesCommonData<'l>
+	fn get_common_data(&self) -> &AxesCommonData
 	{
 		&self.common
 	}
 }
 
-impl<'l> AxesCommon<'l> for Axes2D<'l> {}
+impl AxesCommon for Axes2D {}
 
 pub(crate) trait Axes2DPrivate
 {
 	fn write_out(&self, writer: &mut Writer);
 }
 
-impl<'l> Axes2DPrivate for Axes2D<'l>
+impl Axes2DPrivate for Axes2D
 {
 	fn write_out(&self, writer: &mut Writer)
 	{
